/// @file       webserver2.c
/// @copyright  Copyright (c) 2014 Toradex AG
/// @author     $guc$
/// @date       $2014-2-24$
/// @brief      This code demonstrates how to implement web server on Colibri Module
///             consisting of image and button controls for LEDs. 

#include <windows.h>
#include <stdio.h>
#include <winsock2.h>
#include <gpioLib.h>

#define IP_ADDRESS                                      "0.0.0.0"                             ///< IP address of module
#define PORT_NUMBER                                     10000                                 ///< PORT number used for web server
#define PAGE_SIZE                                       2000                                  ///< HTML page size
#define BUFFER_SIZE                                     255                                   
#define REC_BUFFER                                      512                                   ///< size of receive buffer
#define RES_BUFFER_SIZE                                 10000                                 ///< size of response buffer
#define RES_BUFFER_SIZE_2                               5000                                  
#define COLOUR_BUFFER_SIZE                              20                                    ///< size of colour buffer for button
#define LABEL_BUFFER_SIZE                               4                                     ///< size of label buffer for button
#define GPIO                                            -1

#pragma comment(lib, "Ws2.lib")                                                               ///< linking winsock library

int wmain()
{
    CHAR faviconHeaderString[] = "HTTP/1.1 204 No Content\r\nContent-Length: 0\r\nConnection: keep-alive\r\n\r\n";                                                            ///< "204 No Content" response header string, used for favicon request
    CHAR headerString[] = "HTTP/1.1 200 OK\r\nConnection: keep-alive\r\nContent-type: text/html\r\nContent-Length:%d\r\n\r\n";                                                ///< response header string for web page
    CHAR imageHeaderStringTemporary[] = "HTTP/1.1 200 OK\r\nServer: Microsoft-WinCE/7.00\r\nContent-Length:%d\r\nConnection: keep-alive\r\nContent-type: image/gif\r\n\r\n";  ///< response header string for image
    CHAR imageHeaderString[3000] = {0};                                                       ///< string for final image header
    CHAR finalHeaderString[BUFFER_SIZE] = {0};                                                ///< string for final HTML header
    CHAR buttonColor1[COLOUR_BUFFER_SIZE] = "#CCCCCC";                                        ///< string for button colour
    CHAR buttonColor2[COLOUR_BUFFER_SIZE] = "#CCCCCC";
    CHAR buttonColor3[COLOUR_BUFFER_SIZE] = "#CCCCCC";
    CHAR buttonColor4[COLOUR_BUFFER_SIZE] = "#CCCCCC";
    CHAR buttonString1[LABEL_BUFFER_SIZE] = "OFF";                                            ///< string for button label, NO or OFF
    CHAR buttonString2[LABEL_BUFFER_SIZE] = "OFF";
    CHAR buttonString3[LABEL_BUFFER_SIZE] = "OFF";
    CHAR buttonString4[LABEL_BUFFER_SIZE] = "OFF";
    CHAR greenColor[COLOUR_BUFFER_SIZE] = "#00FF00";
    CHAR greyColor[COLOUR_BUFFER_SIZE] = "#CCCCCC";
    CHAR recBuffer[REC_BUFFER] = {0};                                                         ///< recBuffer to receive request from client
    CHAR recBufferTemporary[REC_BUFFER] = {0};
    CHAR htmlBuffer[RES_BUFFER_SIZE_2] = {0};                                                 ///< buffer to contain HTML page
    CHAR htmlQuery[RES_BUFFER_SIZE_2] = {0};
    CHAR imageBuffer[RES_BUFFER_SIZE] = {0};                                                  ///< buffer to contain image content
    CHAR htmlQueryFinal[RES_BUFFER_SIZE] = {0};                                               ///< string for final response
    BOOL buttonFlag1 = FALSE;                                                                 ///< flags for button state, ON or OFF
    BOOL buttonFlag2 = FALSE;
    BOOL buttonFlag3 = FALSE;
    BOOL buttonFlag4 = FALSE;
    DWORD htmlSize = 0;                                                                       ///< variable for HTML file size
    DWORD imageFileSize = 0;
    DWORD outputPin1 = 45;                                                                    ///< LED 1 connected to SODIMM pin 45 
    DWORD outputPin2 = 55;                                                                    ///< LED 2 connected to SODIMM pin 55 
    DWORD outputPin3 = 63;                                                                    ///< LED 3 connected to SODIMM pin 63 
    DWORD outputPin4 = 100;                                                                   ///< LED 4 connected to SODIMM pin 100
    DWORD temporaryCount = 0;
    INT retVal = 0;
    WSADATA windowsSocket;                                                                    ///< structure contains information about the Windows Sockets implementation
    struct sockaddr_in tServerInfo;                                                           ///< structure contains information about socket address
    SOCKET listeningSocket;                                                                   ///< socket that is bound to a specific transport service provider
    SOCKET client;
    FILE *htmlFile = NULL;                                                                    ///< pointer to HTML file                        
    FILE *imageFile = NULL;                                                                   ///< pointer to image file

    InitGPIOLib();                                                                            ///< initialize GPIO library for module
    /// set SODIMM pin as output
    SetPinAltFn(outputPin1, GPIO, DIR_OUT);
    SetPinAltFn(outputPin2, GPIO, DIR_OUT);
    SetPinAltFn(outputPin3, GPIO, DIR_OUT);
    SetPinAltFn(outputPin4, GPIO, DIR_OUT);

    retVal = WSAStartup(0x0101, &windowsSocket);                                              ///< initiating use of ws2.dll(library' used for socket programming) with winsock Version 1.1
    if (retVal != 0)                                                                          ///< if WSAStartup failed to initialize
    {
        printf("WSAStartup failed with error code: %d\n", retVal);
        exit(1);
    }
    listeningSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);                              ///< socket creation
    if (listeningSocket == INVALID_SOCKET)
    {
        printf("\nsocket creation failed with error code : %d", WSAGetLastError());
        WSACleanup();
        exit(1);
    }

    /// socket binding
    tServerInfo.sin_family = AF_INET;                                                          ///< tcp/udp socket
    tServerInfo.sin_addr.s_addr = inet_addr(IP_ADDRESS);                                       ///< IP address of Colibri module
    tServerInfo.sin_port = htons(PORT_NUMBER);                                                 ///< port number:10000 used for communication 
    retVal = bind(listeningSocket, (SOCKADDR*)&tServerInfo, sizeof(struct sockaddr));
    if (retVal == SOCKET_ERROR)
    {
        printf("\nsocket binding failed with error code : %d", WSAGetLastError());
        WSACleanup();
        exit(1);
    }
    retVal = listen(listeningSocket, 10);                                                      ///< put socket in listening state with 10 maximum connection
    if (retVal == SOCKET_ERROR)
    {
        printf("\nsocket listening failed with error code : %d", WSAGetLastError());
        WSACleanup();
        getchar();
        exit(1);
    }
    printf("socket created.\nWaiting for client :\n");

    while (TRUE)
    {
        client = accept(listeningSocket, NULL, NULL);                                          ///< waiting for request from client
        if (client == INVALID_SOCKET)
        {
            MessageBox(0, L"Could not accept connection request", L"error",0);
            WSACleanup();
            getchar();
            exit(1);
        }
        retVal = recv(client, recBufferTemporary, REC_BUFFER, 0);                              ///< receiving request from client
        recBufferTemporary[retVal] = 0;
        strncpy_s(recBuffer, REC_BUFFER, recBufferTemporary, 50);                              ///< obtain request string

        if (strstr(recBuffer, "GET / HTTP/1.1"))                                               ///< if client request for Web page
        {
            fopen_s(&htmlFile, "\\Program Files\\webserverdemo2\\webpage.htm","r");            ///< open HTML web page
            fseek(htmlFile, 0, SEEK_END);
            htmlSize = ftell(htmlFile);                                                        ///< obtain size of Web page
            fseek(htmlFile, 0, SEEK_SET);
            fread(htmlBuffer, sizeof(CHAR), htmlSize, htmlFile);
            sprintf(htmlQuery, htmlBuffer, buttonColor1, buttonString1, buttonColor2, buttonString2, buttonColor3, buttonString3, buttonColor4, buttonString4);  ///< modify Web page with button colour and label

            sprintf_s(finalHeaderString, BUFFER_SIZE, headerString, strlen(htmlQuery));        ///< modify header string with size of HTML page 
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s%s", finalHeaderString, htmlQuery);  ///< concatenate header and HTML page
            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);                  ///< send final HTML page to client
            fclose(htmlFile);
            /// set output pin High or Low according to respective flag
            SetPinLevel(outputPin1, buttonFlag1);
            SetPinLevel(outputPin2, buttonFlag2);
            SetPinLevel(outputPin3, buttonFlag3);
            SetPinLevel(outputPin4, buttonFlag4);
        }
        else if (strstr(recBuffer, "favicon"))                                                 ///< if client request for "favicon"
        {
            send(client, faviconHeaderString, strlen(faviconHeaderString), 0);                 ///< send "204 No Content" response code to client
        }
        else if (strstr(recBuffer, "ToradexLogo"))                                             ///< if client request for "TroadexLogo"
        {
            fopen_s(&imageFile, "\\Program Files\\webserverdemo2\\toradexlogo.gif","rb");      ///< open image file
            fseek(imageFile, 0, SEEK_END);
            imageFileSize = ftell(imageFile);                                                  ///< obtain Image size
            fseek(imageFile, 0, SEEK_SET);
            fread(imageBuffer, sizeof(BYTE), imageFileSize, imageFile);

            sprintf_s(imageHeaderString, 3000, imageHeaderStringTemporary, imageFileSize);     ///< modify image header with size of Image, 3000 is destination buffer size
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s", imageHeaderString);               ///< Add image header in response string
            for (temporaryCount = strlen(imageHeaderString); temporaryCount < (strlen(imageHeaderString) + imageFileSize); temporaryCount++)
                htmlQueryFinal[temporaryCount] = imageBuffer[temporaryCount - strlen(imageHeaderString)];      ///< concatenate image data in response string

            retVal = send(client, htmlQueryFinal, (strlen(imageHeaderString) + imageFileSize), 0);             ///< send final image response string to client
            fclose(imageFile);
            
        }
        else if (strstr(recBuffer, "Button1.htm"))                                              ///< if client request for response, when button 1 is pressed
        {
            if (buttonFlag1 == TRUE)                                                            ///< if button was ON, switch it OFF
            {
                buttonFlag1 = FALSE;
                strcpy_s(buttonColor1, COLOUR_BUFFER_SIZE, greyColor);
                strcpy_s(buttonString1, LABEL_BUFFER_SIZE, "OFF");
            }
            else                                                                                ///< if button was OFF, switch it ON
            {
                buttonFlag1 = TRUE;
                strcpy_s(buttonColor1, COLOUR_BUFFER_SIZE, greenColor);
                strcpy_s(buttonString1, LABEL_BUFFER_SIZE, "ON");
            }
            fopen_s(&htmlFile, "\\Program Files\\webserverdemo2\\webpage.htm","rb");            ///< open HTML web page
            fseek(htmlFile, 0, SEEK_END);
            htmlSize = ftell(htmlFile);                                                         ///< obtain size of Web page
            fseek(htmlFile, 0, SEEK_SET);
            fread(htmlBuffer, 1, htmlSize, htmlFile);

            sprintf(htmlQuery, htmlBuffer, buttonColor1, buttonString1, buttonColor2, buttonString2, buttonColor3, buttonString3, buttonColor4, buttonString4);  ///< modify Web page with button colour and label

            sprintf_s(finalHeaderString, BUFFER_SIZE, headerString, strlen(htmlQuery));          ///< modify header string with size of HTML page
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s%s", finalHeaderString, htmlQuery);    ///< concatenate header and HTML page
            
            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);                    ///< send final HTML page to client
            fclose(htmlFile);
            /// set output pin High or Low according to flag
            SetPinLevel(outputPin1, buttonFlag1);
            SetPinLevel(outputPin2, buttonFlag2);
            SetPinLevel(outputPin3, buttonFlag3);
            SetPinLevel(outputPin4, buttonFlag4);
        }
        else if (strstr(recBuffer, "Button2.htm"))                                               ///< if client request for response, when button 2 is pressed
        {
            if (buttonFlag2 == TRUE)                                                             ///< if button was ON, switch it OFF
            {
                buttonFlag2 = FALSE;
                strcpy_s(buttonColor2, COLOUR_BUFFER_SIZE, greyColor);
                strcpy_s(buttonString2, LABEL_BUFFER_SIZE, "OFF");
            }
            else                                                                                 ///< if button was OFF, switch it ON
            {
                buttonFlag2 = TRUE;
                strcpy_s(buttonColor2, COLOUR_BUFFER_SIZE, greenColor);
                strcpy_s(buttonString2, LABEL_BUFFER_SIZE, "ON");
            }
            fopen_s(&htmlFile, "\\Program Files\\webserverdemo2\\webpage.htm","rb");              ///< open HTML web page
            fseek(htmlFile, 0, SEEK_END);
            htmlSize = ftell(htmlFile);                                                           ///< obtain size of Web page
            fseek(htmlFile, 0, SEEK_SET);
            fread(htmlBuffer, 1, htmlSize, htmlFile);
            sprintf(htmlQuery, htmlBuffer, buttonColor1, buttonString1, buttonColor2, buttonString2, buttonColor3, buttonString3, buttonColor4, buttonString4);  ///< modify Web page with button colour and label
            
            sprintf_s(finalHeaderString, BUFFER_SIZE, headerString, strlen(htmlQuery));           ///< modify header string with size of HTML page
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s%s", finalHeaderString, htmlQuery);     ///< concatenate header and HTML page

            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);                     ///< send final HTML page to client
            fclose(htmlFile);
            /// set output pin High or Low according to flag
            SetPinLevel(outputPin1, buttonFlag1);
            SetPinLevel(outputPin2, buttonFlag2);
            SetPinLevel(outputPin3, buttonFlag3);
            SetPinLevel(outputPin4, buttonFlag4);
        }
        else if (strstr(recBuffer, "Button3.htm"))                                                 ///< if client request for response, when button 3 is pressed
        {
            if (buttonFlag3 == TRUE)                                                               ///< if button was ON, switch it OFF
            {   
                buttonFlag3 = FALSE;
                strcpy_s(buttonColor3, COLOUR_BUFFER_SIZE, greyColor);
                strcpy_s(buttonString3, LABEL_BUFFER_SIZE, "OFF");
            }
            else                                                                                   ///< if button was OFF, switch it ON
            {
                buttonFlag3 = TRUE;
                strcpy_s(buttonColor3, COLOUR_BUFFER_SIZE, greenColor);
                strcpy_s(buttonString3, LABEL_BUFFER_SIZE, "ON");
            }
            fopen_s(&htmlFile, "\\Program Files\\webserverdemo2\\webpage.htm","rb");               ///< open HTML web page
            fseek(htmlFile, 0, SEEK_END);
            htmlSize = ftell(htmlFile);                                                            ///< obtain size of Web page
            fseek(htmlFile, 0, SEEK_SET);
            fread(htmlBuffer, 1, htmlSize, htmlFile);
            sprintf(htmlQuery, htmlBuffer, buttonColor1, buttonString1, buttonColor2, buttonString2, buttonColor3, buttonString3, buttonColor4, buttonString4);  ///< modify Web page with button colour and label
            
            sprintf_s(finalHeaderString, BUFFER_SIZE, headerString, strlen(htmlQuery));            ///< modify header string with size of HTML page
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s%s", finalHeaderString, htmlQuery);      ///< concatenate header and HTML page

            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);                      ///< send final HTML page to client
            fclose(htmlFile);
            /// set output pin High or Low according to flag
            SetPinLevel(outputPin1, buttonFlag1);
            SetPinLevel(outputPin2, buttonFlag2);
            SetPinLevel(outputPin3, buttonFlag3);
            SetPinLevel(outputPin4, buttonFlag4);
        }
        else if (strstr(recBuffer, "Button4.htm"))                                                  ///< if client request for response, when button 4 is pressed
        {
            if (buttonFlag4 == TRUE)                                                                ///< if button was ON, switch it OFF
            {
                buttonFlag4 = FALSE;
                strcpy_s(buttonColor4, COLOUR_BUFFER_SIZE, greyColor);
                strcpy_s(buttonString4, LABEL_BUFFER_SIZE, "OFF");
            }
            else                                                                                    ///< if button was OFF, switch it ON
            {
                buttonFlag4 = TRUE;
                strcpy_s(buttonColor4, COLOUR_BUFFER_SIZE, greenColor);
                strcpy_s(buttonString4, LABEL_BUFFER_SIZE, "ON");
            }
            fopen_s(&htmlFile, "\\Program Files\\webserverdemo2\\webpage.htm","rb");                ///< open HTML web page
            fseek(htmlFile, 0, SEEK_END);
            htmlSize = ftell(htmlFile);                                                             ///< obtain size of Web page
            fseek(htmlFile, 0, SEEK_SET);
            fread(htmlBuffer, 1, htmlSize, htmlFile);
            sprintf(htmlQuery, htmlBuffer, buttonColor1, buttonString1, buttonColor2, buttonString2, buttonColor3, buttonString3, buttonColor4, buttonString4);  ///< modify Web page with button colour and label
            
            sprintf_s(finalHeaderString, BUFFER_SIZE, headerString, strlen(htmlQuery));             ///< modify header string with size of HTML page
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s%s", finalHeaderString, htmlQuery);       ///< concatenate header and HTML page

            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);                       ///< send final HTML page to client
            fclose(htmlFile);
            /// set output pin High or Low according to flag
            SetPinLevel(outputPin1, buttonFlag1);
            SetPinLevel(outputPin2, buttonFlag2);
            SetPinLevel(outputPin3, buttonFlag3);
            SetPinLevel(outputPin4, buttonFlag4);
        }

        if (retVal == SOCKET_ERROR)
        {
            printf("Error, closing connection");
            MessageBox(0, L"could not send message to client", L"error", 0);
            WSACleanup();
            exit(1);
        } 
        shutdown(client, SD_BOTH);                                                                   ///< closing both sending and receiving of client named socket
        closesocket(client);
    }
     
    closesocket(listeningSocket);                                                                    ///< closing listeningSocket named Socket
    WSACleanup();                                                                                    ///< releasing use of ws2.dll(library' used for socket programming) with winsock Version 1.1
    DeInitGPIOLib();                                                                                 ///< releasing GPIO library
    return 0;
}

